MODULE sbcwave
   !!======================================================================
   !!                       ***  MODULE  sbcwave  ***
   !! Wave module 
   !!======================================================================
   !! History :  3.3.1  !   2011-09  (Adani M)  Original code
   !!----------------------------------------------------------------------
   USE iom             ! I/O manager library
   USE in_out_manager  ! I/O manager
   USE lib_mpp         ! distribued memory computing library
   USE fldread         ! read input fields
   USE sbc_oce         ! Surface boundary condition: ocean fields
   USE phycst          ! physical constants
   ! USE sbcblk_core, ONLY: Cd_n10   
   !!----------------------------------------------------------------------
   !!   sbc_wave       : read drag coefficient from wave model in netcdf files 
   !!----------------------------------------------------------------------

   IMPLICIT NONE
   PRIVATE

   PUBLIC   sbc_wavepar    ! routine called in sbc_blk_core or sbc_blk_mfs
    PUBLIC   sbc_wave    ! routine called in sbc_blk_core or sbc_blk_mfs

   INTEGER , PARAMETER ::   jpfld_wavepar = 7    ! maximum number of fields to be read 
   INTEGER , PARAMETER ::   jp_ust = 1           ! index of Stokes velocity (east component) (m/s)  at T-point
   INTEGER , PARAMETER ::   jp_vst = 2           ! index of Stokes velocity (north component) (m/s) at T-point
   INTEGER , PARAMETER ::   jp_swh = 3           ! index of significant wave height (m)
   INTEGER , PARAMETER ::   jp_mwp = 4           ! index of mean wave period (s)
   INTEGER , PARAMETER ::   jp_phioc = 5         ! index of normalized energy flux into the ocean (non-dim)
   INTEGER , PARAMETER ::   jp_tauoc = 6         ! index of normalized wave stress into the ocean (non-dim)
   !INTEGER , PARAMETER ::   jp_cdww = 7          ! index of wave drag coeff (non-dim)
   INTEGER , PARAMETER ::   jp_wspd = 7          ! index of 10 m neutral wind speed (m/s)

   TYPE(FLD), ALLOCATABLE, DIMENSION(:)  :: sf_wave ! structure of input fields (file informations, fields read)
   !REAL(wp),PUBLIC,ALLOCATABLE,DIMENSION (:,:)       :: cdn_wave ! CCC moved to sbc_oce
   TYPE(FLD), ALLOCATABLE, DIMENSION(:)  :: sf_wavepar ! structure of input fields (file informations, fields read)
   REAL(wp),PUBLIC,ALLOCATABLE,DIMENSION (:,:)       :: ust_wavepar, vst_wavepar, swh_wavepar, mwp_wavepar
   REAL(wp),PUBLIC,ALLOCATABLE,DIMENSION (:,:)       :: phioc_wavepar  !WAVE2NEMO 14.12.2016 moved rn_crban to sbc_oce
   !REAL(wp),PUBLIC,ALLOCATABLE,DIMENSION (:,:)       :: tauoc_wavepar ! CCC moved to sbc_oce
   !REAL(wp),PUBLIC,ALLOCATABLE,DIMENSION (:,:)       :: wspd_wavepar  ! CCC moved to sbc_oce
   !REAL(wp),PUBLIC,ALLOCATABLE,DIMENSION (:,:)       :: cdww_wavepar

   !! * Substitutions
#  include "domzgr_substitute.h90"
#  include "vectopt_loop_substitute.h90"
   !!----------------------------------------------------------------------
   !! NEMO/OPA 4.0 , NEMO Consortium (2011) 
   !! $Id$
   !! Software governed by the CeCILL licence     (NEMOGCM/NEMO_CeCILL.txt)
   !!----------------------------------------------------------------------
CONTAINS

   SUBROUTINE sbc_wave( kt )
      !!---------------------------------------------------------------------
      !!                     ***  ROUTINE sbc_apr  ***
      !!
      !! ** Purpose :   read drag coefficient from wave model  in netcdf files.
      !!
      !! ** Method  : - Read namelist namsbc_wave
      !!              - Read Cd_n10 fields in netcdf files
      !! ** action  :
      !!
      !!---------------------------------------------------------------------
      INTEGER, INTENT( in  ) ::  kt   ! ocean time step
      INTEGER                ::  ierror   ! return error code
      CHARACTER(len=100)     ::  cn_dir_cdg                      ! Rootdirectory for location of drag coefficient files
      TYPE(FLD_N)            ::  sn_cdg                          ! informationsabout the fields to be read
      !!---------------------------------------------------------------------
      NAMELIST/namsbc_wave/  sn_cdg, cn_dir_cdg
      !!---------------------------------------------------------------------

      !!----------------------------------------------------------------------
      !
      !
      !                                         ! -------------------- !
      IF( kt == nit000 ) THEN                   ! First call kt=nit000 !
         !                                      ! -------------------- !
         !                                            !* set file information
         !                                            (default values)
         ! ... default values (NB: frequency positive => hours, negative =>
         ! months)
         !              !   file   ! frequency !  variable  ! time intep !  clim
         !              ! 'yearly' or ! weights  ! rotation !
         !              !   name   !  (hours)  !   name     !   (T/F)    !
         !              (T/F)  !  'monthly'  ! filename ! pairs    !
         sn_cdg = FLD_N('cdg_wave'  ,    1     ,'drag_coeff',  .true.    ,.false. ,   'daily'   , ''       , '',''       )
         cn_dir_cdg = './'          ! directory in which the Patm data are


         REWIND( numnam_ref )                             !* read in namlist namsbc_wave
         READ  ( numnam_ref, namsbc_wave )
	 REWIND( numnam_ref )
         REWIND( numnam_cfg )                             !* read in namlist namsbc_wave
         READ  ( numnam_cfg, namsbc_wave )
	 REWIND( numnam_cfg )
         !

         ALLOCATE( sf_wave(1), STAT=ierror )           !* allocate and fillsf_wave with sn_cdg
         IF( ierror > 0 )   CALL ctl_stop( 'STOP', 'sbc_wave: unable to allocatesf_wave structure' )
         !
         CALL fld_fill( sf_wave, (/ sn_cdg /), cn_dir_cdg, 'sbc_wave', 'Wavemodule ', 'namsbc_wave' )
                                ALLOCATE( sf_wave(1)%fnow(jpi,jpj,1)   )
         IF( sn_cdg%ln_tint )   ALLOCATE( sf_wave(1)%fdta(jpi,jpj,1,2) )
         !ALLOCATE( cdn_wave(jpi,jpj) )
         ! Allocation done by sbc_oce
         cdn_wave(:,:) = 0.0
      ENDIF
         !
         !
      CALL fld_read( kt, nn_fsbc, sf_wave )      !* read drag coefficient fromexternal forcing
      cdn_wave(:,:) = sf_wave(1)%fnow(:,:,1)
   END SUBROUTINE sbc_wave


   SUBROUTINE sbc_wavepar( kt )
      !!---------------------------------------------------------------------
      !!                     ***  ROUTINE sbc_wavepar  ***
      !!
      !! ** Purpose :   Provide at each time step wave model parameters, including the
      !!                Stokes drift (east and north components), significant wave height and the
      !!                mean wave period as well as the normalized stress and energy flux into the 
      !!                ocean for TKE.
      !!
      !!
      !! ** Method  : - Read namelist namsbc_wavepar
      !!              - Read fields in NetCDF files 
      !! ** action  :   
      !!               
      !!---------------------------------------------------------------------
      INTEGER, INTENT( in  ) ::  kt  ! ocean time step
      !!
      INTEGER  ::  ierror ! return error code
      INTEGER  ::   ji,iii,jjj      ! dummy loop index, Victor added iii, jjj
      INTEGER  ::   jfld    ! dummy loop arguments
      !!
      CHARACTER(len=100)     ::  cn_dir_wavepar ! Root directory for location of ECWAM wave parameter fields
      TYPE(FLD_N), DIMENSION(jpfld_wavepar) :: slf_i ! array of namelist informations on the fields to read
      TYPE(FLD_N)            ::  sn_ust, sn_vst, sn_swh, sn_mwp ! information about the fields to be read
      TYPE(FLD_N)            ::  sn_phioc, sn_tauoc
      TYPE(FLD_N)            ::  sn_wspd
      !TYPE(FLD_N)            ::  sn_cdww
      !!---------------------------------------------------------------------
      !NAMELIST/namsbc_wavepar/  sn_ust, sn_vst, sn_swh, sn_mwp, sn_phioc, sn_tauoc, cn_dir_wavepar
      NAMELIST/namsbc_wavepar/  sn_ust, sn_vst, sn_swh, sn_mwp, sn_wspd, sn_phioc, sn_tauoc, cn_dir_wavepar
      !!---------------------------------------------------------------------

      !!----------------------------------------------------------------------
      !
      !
      !                                         ! -------------------- !
      IF( kt == nit000 ) THEN                   ! First call kt=nit000 !
         !                                      ! -------------------- !
         ! set file information (default values)
         ! ... default values (NB: frequency positive => hours, negative => months)
         !            !   file   ! frequency !  variable  ! time intep !  clim   ! 'yearly' or ! weights  ! rotation !
         !            !   name   !  (hours)  !   name     !   (T/F)    !  (T/F)  !  'monthly'  ! filename ! pairs    !
         sn_ust = FLD_N( 'ust'   ,     6     ,  'ust'     ,  .true.    , .false. ,   'monthly' , ''       , '' ,''       )
         sn_vst = FLD_N( 'vst'   ,     6     ,  'vst'     ,  .true.    , .false. ,   'monthly' , ''       , '' ,''      )
         sn_swh = FLD_N( 'swh'   ,     6     ,  'swh'     ,  .true.    , .false. ,   'monthly' , ''       , '' ,''      )
         sn_mwp = FLD_N( 'mwp'   ,     6     ,  'mwp'     ,  .true.    , .false. ,   'monthly' , ''       , '' ,''      )
         !sn_cdww = FLD_N( 'cdww'  ,    6     ,  'cdww'    ,  .true.    , .false. ,   'monthly' , ''       , ''       )
         sn_wspd = FLD_N( 'wspd'   ,   6     ,  'wspd'    ,  .true.    , .false. ,   'monthly' , ''       , '',''       )
         sn_phioc = FLD_N( 'phioc'   , 6     ,  'phioc'   ,  .true.    , .false. ,   'monthly' , ''       , '',''       )
         sn_tauoc = FLD_N( 'tauoc'   , 6     ,  'tauoc'   ,  .true.    , .false. ,   'monthly' , ''       , '',''       )
         cn_dir_wavepar = './'          ! directory in which the wave data are found

         REWIND( numnam_ref )                             !* read in namlist namsbc_wave
         READ  ( numnam_ref, namsbc_wavepar ) 
         REWIND ( numnam_ref )

         REWIND( numnam_cfg )                             !* read in namlist namsbc_wave
         READ  ( numnam_cfg, namsbc_wavepar ) 
         REWIND ( numnam_cfg )
         !


         slf_i(jp_ust) = sn_ust 
         slf_i(jp_vst) = sn_vst
         slf_i(jp_swh) = sn_swh
         slf_i(jp_mwp) = sn_mwp
         !slf_i(jp_cdww) = sn_cdww
         slf_i(jp_wspd) = sn_wspd
         slf_i(jp_phioc) = sn_phioc
         slf_i(jp_tauoc) = sn_tauoc

         ALLOCATE( sf_wavepar(jpfld_wavepar), STAT=ierror )          !* set sf_wavepar structure
         IF ( ierror > 0 ) THEN
            CALL ctl_stop( 'STOP', 'sbc_wavepar: unable to allocate sf_wavepar structure' ) ; RETURN
         ENDIF
         !
         jfld = jpfld_wavepar
         DO ji = 1, jpfld_wavepar
            ALLOCATE( sf_wavepar(ji)%fnow(jpi,jpj,1)   )
            IF ( slf_i(ji)%ln_tint ) ALLOCATE( sf_wavepar(ji)%fdta(jpi,jpj,1,2) )
         ENDDO
         ALLOCATE( ust_wavepar(jpi,jpj) )
         ALLOCATE( vst_wavepar(jpi,jpj) )
         ALLOCATE( swh_wavepar(jpi,jpj) )
         ALLOCATE( mwp_wavepar(jpi,jpj) )
         !ALLOCATE( cdww_wavepar(jpi,jpj) )
         !ALLOCATE( wspd_wavepar(jpi,jpj) )
         ALLOCATE( phioc_wavepar(jpi,jpj) )
         ALLOCATE( tauoc_wavepar(jpi,jpj) )
         !
         CALL fld_fill( sf_wavepar, slf_i, cn_dir_wavepar, 'sbc_wavepar', 'Wave module ', 'namsbc_wavepar' )
         !
      ENDIF
         !
      CALL fld_read( kt, nn_fsbc, sf_wavepar )      !* read wave parameters from ECWAM NetCDF file
      ust_wavepar(:,:) = sf_wavepar(jp_ust)%fnow(:,:,1)
      vst_wavepar(:,:) = sf_wavepar(jp_vst)%fnow(:,:,1)
      swh_wavepar(:,:) = sf_wavepar(jp_swh)%fnow(:,:,1)
      mwp_wavepar(:,:) = sf_wavepar(jp_mwp)%fnow(:,:,1)
      !cdww_wavepar(:,:) = sf_wavepar(jp_cdww)%fnow(:,:,1)
      wspd_wavepar(:,:) = sf_wavepar(jp_wspd)%fnow(:,:,1)
      phioc_wavepar(:,:) = sf_wavepar(jp_phioc)%fnow(:,:,1)
      tauoc_wavepar(:,:) = sf_wavepar(jp_tauoc)%fnow(:,:,1)
 
     
       rn_crban(:,:)=29*phioc_wavepar(:,:) ! Alfa is phioc*sqrt(rw/ra)sbc_wa
       ! Limit cr_ban between 10 and 300
       DO iii=1,jpi
           DO jjj=1,jpj

                IF (rn_crban(iii,jjj) < 10 ) THEN
                rn_crban(iii,jjj)=10                
                ELSEIF (rn_crban(iii,jjj) > 300) THEN               
                rn_crban(iii,jjj)=300
                ENDIF
        
           ENDDO
       ENDDO
 
     END SUBROUTINE sbc_wavepar
      

END MODULE sbcwave
